-- RR4 Adder Module

library ieee;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package adder_pckg is
     component RR4_adder is

	port(
		Scp: in bit;
		Xs, X1, X0: in bit;
		Ys, Y1, Y0: in bit;
		
		Sc, C0, C1: out bit;  -- intermediate carry digit
		Ss, S0, S1: out bit   -- intermediate sum digit
		
	);
     end component RR4_adder;
end adder_pckg;


library ieee;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity RR4_adder is

	port(
		Scp: in bit;
		Xs, X1, X0: in bit;
		Ys, Y1, Y0: in bit;
		
		Sc, C0, C1: out bit;  -- intermediate carry digit
		Ss, S0, S1: out bit   -- intermediate sum digit
				
	);

end entity RR4_adder;

architecture behav of RR4_adder is
	
	begin

-------------------------------------------------------------------------------------------------
--           Intermediate Carry Digit
-------------------------------------------------------------------------------------------------

	     Sc<= ((not Scp) and ((Xs and X1 and Ys and Y1) or (Xs and X1 and X0 and Ys and Y0) 
		  or (Xs and X0 and Ys and Y1 and Y0))) or (Scp and (((not Xs) and(not X1) 
		  and (Ys) and (Y1)) or ((not Xs) and (not X1) and (not X0) and Ys and (not Y1)) 
		  or (X1 and (not X0) and Ys and Y1 and Y0) or (Xs and X1 and Y1)
		  or (Xs and Ys and Y1 and (not Y0)) or (Xs and X1 and X0 and Y1 and (not Y0)) 
		  or (Xs and X1 and X0 and Ys and Y0) or (Xs and X0 and Ys and Y1 and Y0 ) 
		  or (Xs and (not X1) and X0 and (not Y1) and (not Y0)) or (Xs and (not X1) 
		  and Ys and (not Y1) and Y0)));


	     C0<= ((not Scp) and (((not Xs) and X1 and X0 and (not Ys) and Y1) or ((not Xs) 
		  and X1 and (not Ys) and Y1 and Y0))) or (Scp and ((Xs and X1 and X0 and Ys and Y1) 
		  or (Xs and X1 and Ys and Y1 and Y0)));

	     C1<= ((not Scp) and (((not Xs) and X1 and X0 and Ys and Y1 and (not Y0)) or 
		  ((not Xs) and X1 and (not X0) and (not Ys) and Y1 and (not Y0)) or (Xs and X1 
		  and (not X0) and (not Ys) and Y1 and Y0) or ((not Xs) and (not Ys) and (not Y1)
		  and Y0) or ((not Xs) and (not X1) and X0 and (not Y1) and (not Y0)) or 
		  ((not Xs) and X1 and (not Ys) and (not Y1) and (not Y0)) or ((not X1) and 
		  (not Ys) and Y1) or ((not Xs) and X1 and Ys and (not Y1)) or (Xs and X1 and Ys 
		  and Y1) or (Xs and X1 and X0 and Ys and Y0) or (Xs and X0 and Ys and Y1 and Y0)
		  )) or (Scp and ((not Xs) and (not X1) and (not X0) and Ys and (not Y1))) or 
		  ((not Xs) and (not X1) and Ys and Y1) or ((not Xs) and X0 and (not Ys) and Y1 
		  and Y0) or ((not Xs) and X1 and X0 and (not Ys) and Y0) or ((not Xs) and X1 and
		  (not Ys) and Y1) or ((not Xs) and X1 and (not X0) and Y1 and Y0) or (Xs and X1 
		  and (not Ys) and (not Y1)) or (Xs and X1 and (not X0) and Ys and Y1 and 
		  (not Y0)) or (Xs and Ys and (not Y1) and Y0) or (Xs and X1 and X0 and (not Ys) 
		  and (not Y0)) or (Xs and (not X1) and (not Ys) and (not Y1) and (not Y0)) or 
		  (Xs and (not X1) and Ys and Y1);

-------------------------------------------------------------------------------------------------
--            Intermediate Sum Digit
-------------------------------------------------------------------------------------------------
	     
	     Ss<= (not Scp) and (((not Xs) and (not X1) and (not X0) and Y0) or ((not Xs) and 
		  (not X1) and (not X0) and Y1) or ((not Xs) and (not X1) and X0 and (not Ys) 
		  and (not Y1)) or ((not Xs) and X0 and Y1 and (not Y0)) or ((not Xs) and 
		  (not X1) and Ys and Y1 and Y0) or ((not Xs) and X0 and (not Ys) and (not Y1) 
		  and (not Y0)) or ((not Xs) and X1 and X0 and (not Ys) and Y1) or (X1 and 
		  (not X0) and Y1) or ((not Xs) and X1 and Ys and (not Y1) and Y0) or (Xs and X1 
		  and X0 and (not Ys) and (not Y1)) or (Xs and X0 and Y1 and (not Y0)) or 
		  (Xs and X1 and X0 and Ys and Y1) or (Xs and X1 and (not Ys) and (not Y1) and 
		  (not Y0)) or (Xs and (not X1) and X0 and Ys and (not Y1)));
	         
	         
	     S0<= ((not Scp) and (((not Xs) and (not X1) and (not Ys) and (not Y1) and Y0) 
	          or ((not Xs) and (not X1) and Ys and Y1 and Y0) or ((not Xs) and (not X1) and 
	          (not X0) and Y1 and (not Y0)) or ((not Xs) and (not X1) and X0 and (not Y1) and
	          (not Y0)) or ((not Xs) and X1 and X0 and (not Ys) and Y1) or ((not Xs) and X1 
	          and X0 and (not Y0) and Y1) or ((not Xs) and X1 and X0 and Ys and (not Y1)) or
	          (X1 and (not X0) and (not Ys) and (not Y1) and (not Y0)) or (X1 and (not X0) 
	          and (not Ys) and Y1 and Y0) or (X1 and (not X0) and Ys and (not Y1) and Y0) 
	          or (Xs and X1 and X0 and (not Ys) and (not Y1)) or (Xs and (not X1) and X0 
	          and Y1 and (not Y0)) or (Xs and (not X1) and X0 and Ys and (not Y1)) or (Xs 
	          and (not X1) and (not Ys) and Y1 and Y0))) or (Scp and (((not Xs) and (not X1) 
	          and (not X0) and (not Ys) and Y1) or ((not Xs) and (not X1) and Y1 and 
	          (not Y0)) or ((not Xs) and (not X1) and (not X0) and Ys and (not Y1)) or 
	          ((not Xs) and (not X1) and X0 and (not Ys) and (not Y1) and Y0) or ((not Xs) 
	          and (not X1) and X0 and Ys and Y1) or ((not Xs) and X1 and (not Ys) and 
	          (not Y1) and (not Y0)) or ((not Xs) and X1 and X0 and (not Ys) and Y1 and Y0)
	          or ((not Xs) and X1 and X0 and Ys and (not Y1)) or (X1 and (not X0) and 
	          (not Ys) and (not Y1)) or (X1 and (not X0) and Ys and Y1 and Y0) or (Xs and 
	          X1 and (not Ys) and (not Y1) and Y0) or (Xs and X1 and X0 and Y1 and 
	          (not Y0)) or (Xs and X1 and X0 and Ys and Y1) or (Xs and (not X1) and X0 
	          and Ys and (not Y1)) or (Xs and (not X1) and (not Ys) and (not Y1) and 
	          (not Y0)) or (Xs and (not X1) and (not Ys) and Y1 and Y0)));  
	     
	     
	     S1<= ((not Scp) and (((not Xs) and (not X1) and (not X0) and Y0) or (X0 and 
	          (not Y0) and (not Ys) and (not Y1)) or (X0 and (not Y0) and Y1) or (X1 and
	          (not X0) and Y0))) or (Scp and (((not Xs) and (not X1) and (not X0) and 
	          (not Ys) and Y1) or ((not Xs) and (not X1) and (not X0) and Ys and Y0) or 
	          ((not Xs) and X0 and (not Ys) and (not Y1) and (not Y0)) or ((not Xs)and 
	          X0 and Y1 and (not Y0)) or (X1 and (not X0) and Y0) or (Xs and X0 and 
	          (not Ys) and (not Y1) and (not Y0)) or (Xs and X0 and Y1 and (not Y0))));
	          		
end architecture behav;	
