-- RR4 to Binary Conversion

library ieee;
library binrr4;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use binrr4.fa_pckg.all;

package RR4bin_pckg is
	component RR4bin is
	
	port( 
	      S: in std_logic_vector(7 downto 0);   -- sign bits of RR4 digits
	      A1: in std_logic_vector(7 downto 0);  -- higher bits of RR4 digits
	      A0: in std_logic_vector(7 downto 0);  -- lower bits of RR4 digits
	    
	      Bl: out std_logic_vector(7 downto 0); -- output binary register (lower)	      
	      Bh: out std_logic_vector(7 downto 0)  -- output binary register (higher)

	);
	end component;
end package RR4bin_pckg;


library ieee;
library binrr4;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use binrr4.fa_pckg.all;

entity RR4bin is
	
	port( 
	      S: in std_logic_vector(7 downto 0);   -- sign bits of RR4 digits
	      A1: in std_logic_vector(7 downto 0);  -- higher bits of RR4 digits
	      A0: in std_logic_vector(7 downto 0);  -- lower bits of RR4 digits
	    
	      Bl: out std_logic_vector(7 downto 0); -- output binary register (lower)	      
	      Bh: out std_logic_vector(7 downto 0)  -- output binary register (higher)

	    );

end entity RR4bin;


architecture functional of RR4bin is

	signal P0,P1,N0,N1: std_logic_vector(7 downto 0);

	signal C: std_logic_vector(16 downto 0);
        
	begin

	Implement: process(P0,N0,P1,N1) is
	
		begin

		C(0)<='1';

		for k in 0 to 7 loop

			P0(k)<= (not S(k)) and (A0(k));

			P1(k)<= (not S(k)) and  A1(k);
	
			N0(k)<= (not A0(k)) or (not S(k));

			N1(k)<= (not A1(k)) or (not S(k));

		end loop;	

	end process Implement;


	-- storage in binary lower order register
	
 	bit0: full_adder
			
		port map(A=>P0(0), B=>N0(0), Cin=>C(0), S=>Bl(0), Cout=>C(1));

	bit1: full_adder

		port map(P1(0),N1(0),C(1),Bl(1),C(2));

	bit2: full_adder

		port map(P0(1),N0(1),C(2),Bl(2),C(3));

	bit3: full_adder

		port map(P1(1),N1(1),C(3),Bl(3),C(4));

	bit4: full_adder

		port map(P0(2),N0(2),C(4),Bl(4),C(5));

	bit5: full_adder

		port map(P1(2),N1(2),C(5),Bl(5),C(6));

	bit6: full_adder

		port map(P0(3),N0(3),C(6),Bl(6),C(7));

	bit7: full_adder

		port map(P1(3),N1(3),C(7),Bl(7),C(8));



	-- storage in binary higher order output register (Bh)


	bit8: full_adder
			
		port map(P0(4),N0(4),C(8),Bh(0),C(9));

	bit9: full_adder

		port map(P1(4),N1(4),C(9),Bh(1),C(10));

	bit10: full_adder

		port map(P0(5),N0(5),C(10),Bh(2),C(11));

	bi11: full_adder

		port map(P1(5),N1(5),C(11),Bh(3),C(12));

	bit12: full_adder

		port map(P0(6),N0(6),C(12),Bh(4),C(13));

	bit13:full_adder

		port map(P1(6),N1(6),C(13),Bh(5),C(14));

	bit14: full_adder

		port map(P0(7),N0(7),C(14),Bh(6),C(15));

	bit15: full_adder

		port map(P1(7),N1(7),C(15),Bh(7),C(16));
		
		
end architecture functional;
