
-- top level view of the RR-4 Coprocessor


library IEEE;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity system is

	port(
		rst: in std_logic;
		clk: in std_logic	
            );
end entity;

architecture behav of system is

use work.arith_pckg.all;      -- components of RR4 arithmetic unit module
use work.lu_pckg.all;         -- components of logic unit module
use work.decod_pckg.all;      -- components of control unit module
use work.adder_pckg.all;      -- components of RR4_adder module
use work.multiplier_pckg.all; -- components of RR4_multiplier module
use work.parpro_pckg.all;     -- components of partial product module
use work.binRR4_pckg.all;     -- components of binary to RR4 conversion unit
use work.RR4bin_pckg.all;     -- components of RR4 to binary conversion unit
use work.intf_pckg.all;       -- components of bidirectional interface unit

signal temp_ir: std_logic_vector(7 downto 0);
signal op1: std_logic_vector(7 downto 0);
signal op2: std_logic_vector(7 downto 0);
signal Rs1: STD_LOGIC_VECTOR(3 downto 0);
signal R11: STD_LOGIC_VECTOR(3 downto 0);
signal R01: STD_LOGIC_VECTOR(3 downto 0);
signal Rs2: STD_LOGIC_VECTOR(3 downto 0);
signal R12: STD_LOGIC_VECTOR(3 downto 0);
signal R02: STD_LOGIC_VECTOR(3 downto 0); 
signal temp_y0, temp_y1, temp_y2, temp_y3, temp_y4, temp_y5, temp_y6, temp_y7: bit; 
signal temp_T: std_logic_vector(7 downto 0); 
signal temp_Ts: std_logic_vector(4 downto 0);
signal temp_T1: std_logic_vector(4 downto 0);
signal temp_T0: std_logic_vector(4 downto 0);
signal temp_Ms: std_logic_vector(7 downto 0);
signal temp_M1: std_logic_vector(7 downto 0);
signal temp_M0: std_logic_vector(7 downto 0);
signal temp_Bl: std_logic_vector(7 downto 0);
signal temp_Bh: std_logic_vector(7 downto 0);

begin

glue_logic: process is

begin

rst:='0';

ir_trans: component RegSinc
		  generic map(8)
		  port map(
			    clk=>clk,
			    reset=>rst,
			    load=>'1',
			    temp_ir(7 downto 0)=>r0(7 downto 0),
			    dout(7downto 0)=>00000000
		  );

op1_trans: RegSinc
		  generic map(8)
		  port map(
			    clk=>clk,
			    reset=>rst,
			    load=>'1',
			    op1(7 downto 0)=>r1(7 downto 0),
			    dout(7downto 0)=>"00000000"
		  );

op2_trans: RegSinc
		  generic map(8)
		  port map(
			    clk=>clk,
			    reset=>rst,
			    load=>'1',
			    op2(7 downto 0)=>r2(7 downto 0),
			    dout(7downto 0)=>"00000000"
		  );

if rst='0' then

	ir_transfer: decoder 
			port map(
				 enable=>rst, 
				 IR(7 downto 0)=>temp_ir(7 downto 0),
				 y0=>temp_y0, 
				 y1=>temp_y1, 
				 y2=>temp_y2, 
				 y3=>temp_y3, 
				 y4=>temp_y4, 
				 y5=>temp_y5, 
				 y6=>temp_y6, 
				 y7=>temp_y7
			);  

	if IR(7)='0' then

		op_transfer1(AU): binRR4
				     port map( 
					       B(7 downto 0)=>op1(7 downto 0), 
					       Rs(3 downto 0)=>Rs1(3 downto 0),
					       R1(3 downto 0)=>R11(3 downto 0), 
					       R0(3 downto 0)=>R01(3 downto 0)
				     ); 

		op_transfer2(AU): binRR4
				     port map( 
					       B(7 downto 0)=>op2(N-1 downto 0), 
					       Rs(3 downto 0)=>Rs2(3 downto 0),
					       R1(3 downto 0)=>R12(3 downto 0), 
					       R0(3 downto 0)=>R02(3 downto 0)
				     ); 

	elsif

		op_transfer(LU): logic_unit
				     port map(
					       y4=>temp_y4, 
					       y5=>temp_y5, 
					       y6=>temp_y6,
					       y7=>temp_y7,
					       L0(7 downto 0)=>op1(N-1 downto 0),
					       L1(7 downto 0)=>op2(N-1 downto 0),
					       T(7 downto 0)=>temp_T(7 downto 0)
					);

	end if;

		
	arith_op: arithmetic_unit
			port map(
				  y0=>temp_y0,
				  y1=>temp_y1,
			          y2=>temp_y2,
				  y3=>temp_y3,
				  As(3 downto 0)=>Rs1(3 downto 0),
				  A1(3 downto 0)=>R11(3 downto 0),
			          A0(3 downto 0)=>R01(3 downto 0),
			          Bs(3 downto 0)=>Rs2(3 downto 0),
				  B1(3 downto 0)=>R12(3 downto 0),
				  B0(3 downto 0)=>R02(3 downto 0),
				  Ts(4 downto 0)=>temp_Ts(4 downto 0),
				  T1(4 downto 0)=>temp_T1(4 downto 0),
				  T0(4 downto 0)=>temp_T0(4 downto 0),
				  Ms(7 downto 0)=>temp_Ms(7 downto 0),
				  M1(7 downto 0)=>temp_M1(7 downto 0),
				  M0(7 downto 0)=>temp_M0(7 downto 0)      
			);

	if temp_y2:='1' then

		reconv1: RR4bin
			   	port map(
					  S(7 downto 0)=>temp_Ms(7 downto 0),
					  A1(7 downto 0)=>temp_M1(7 downto 0),
					  A0(7 downto 0)=>temp_M0(7 downto 0),
					  Bl(7 downto 0)=>temp_Bl(7 downto 0),
					  Bh(7 downto 0)=>temp_Bh(7 downto 0)
				);

	elsif

		reconv2: RR4bin
				port map(
					  S(7 downto 5)=>"000",
					  S(4 downto 0)=>temp_Ts(4 downto 0),
					  A1(7 downto 5)=>"000",
					  A1(4 downto 0)=>temp_Ts(4 downto 0),
					  A0(7 downto 5)=>"000",
					  S(4 downto 0)=>temp_Ts(4 downto 0),			
					  Bl(7 downto 0)=>temp_Bl(7 downto 0),
					  Bh(7 downto 0)=>temp_Bh(7 downto 0)
				);
	end if;

	rst:='1';

	if temp_ir(7)='0' then

		output_trans1(A): RegSinc
		  	generic map(8)
		  	port map(
			    	  clk=>clk,
			    	  reset=>rst,
			    	  load=>'0',
			    	  din(7 downto 0)=>"00000000",
			      	  r1(7 downto 0)=>Bl(7 downto 0),
		  	); 		

		output_trans2(B): RegSinc
		  	generic map(8)
		  	port map(
			    	  clk=>clk,
			    	  reset=>rst,
			    	  load=>'0',
			    	  din(7 downto 0)=>"00000000",
			      	  r2(7 downto 0)=>Bh(7 downto 0),
		  	);

	else

		output_trans(L): RegSinc
		  	generic map(8)
		  	port map(
			    	  clk=>clk,
			    	  reset=>rst,
			    	  load=>'0',
			    	  din(7 downto 0)=>"00000000",
			      	  r3(7 downto 0)=>temp_T(7 downto 0),
		  	);

	end if;

	end process glue_logic;

end architecture behav;

